////////////////////////////////////////////////////////////////////////////////
// TTransBitmap component
//
// Controls that allows to display a button like with a transparent bitmap, a
// caption, etc. Different ways of displaying both the caption and the picture.
// And... lots of other facilities... :-)
//
// by Alan GARNY
//    gry@physiol.ox.ac.uk
//    http://pc-heartbreak.physiol.ox.ac.uk/
//
//  Copyright 1997-98
////////////////////////////////////////////////////////////////////////////////

#ifndef TransBitmapH
#define TransBitmapH

////////////////////////////////////////////////////////////////////////////////

enum TTransBitmapBitmapDisplay {bdTopLeft, bdTopCenter, bdTopRight,
                                bdMiddleLeft, bdMiddleCenter, bdMiddleRight,
                                bdBottomLeft, bdBottomCenter, bdBottomRight};
enum TTransBitmapBitmapNumber {bn1, bn2, bn3, bn4};
enum TTransBitmapCaptionDisplay {cdBitmapLeft, cdBitmapRight, cdBitmapTop,
                                 cdBitmapBottom, cdTopLeft, cdTopCenter,
                                 cdTopRight, cdMiddleLeft, cdMiddleCenter,
                                 cdMiddleRight, cdBottomLeft, cdBottomCenter,
                                 cdBottomRight};
enum TTransBitmapLook {loButton, loPanel, loClickablePanel};
enum TTransBitmapState {stUp, stDown, stDisabled};
enum TTransBitmapAutoSize {asWidthOnly, asHeightOnly, asBoth};
enum TTransBitmapGradientType {grLeftToRight, grRightToLeft,
                               grTopToBottom, grBottomToTop,
                               grHorizCenterToBorder, grHorizBorderToCenter,
                               grVertCenterToBorder, grVertBorderToCenter};

////////////////////////////////////////////////////////////////////////////////

#if (__BORLANDC__ >= 0x0530)   // i.e. NOT BCB 1.0
class PACKAGE TTransBitmap: public TGraphicControl
#else
class TTransBitmap: public TGraphicControl
#endif
{
   private:
      // Private representation of published properties

      TAlignment FAlignment;
      bool FAllowAllUp;
      bool FAllowDown;
      bool FAllowFocus;
      bool FAllowFullTransparency;
      bool FAutoRefresh;
      bool FAllowSubAndSuperChars;
      bool FAutoSize;
      TTransBitmapAutoSize FAutoSizeParams;
      TColor FBackgroundColor;
      TPanelBevel FBevelInner;
      TPanelBevel FBevelOuter;
      int FBevelWidth;
      Graphics::TBitmap *FBitmap;
      TTransBitmapBitmapDisplay FBitmapDisplay;
      TTransBitmapBitmapNumber FBitmapNumber;
      TBorderWidth FBorderWidth;
      TTransBitmapCaptionDisplay FCaptionDisplay;
      int FCaptionNbLines;
      int FCaptionToBitmapDist;
      String FEscapeChar;
      TColor FFocusColor;
      int FFocusWidth;
      TColor FGradientColor;
      bool FGradientEnabled;
      int FGradientNbOfColors;
      TTransBitmapGradientType FGradientType;
      int FGroupIndex;
      bool FHasFocus;
      TTransBitmapLook FLook;
      TTransBitmapState FState;
      bool FStretch;
      String FSubChar;
      int FSubCharDist;
      int FSubFontSizeDiff;
      String FSuperChar;
      int FSuperCharDist;
      int FSuperFontSizeDiff;

      // Properties used for internal purpose

      bool FirstTime;

      int BitmapWidth;
      int CaptionWidth, CaptionHeight;
      TStringList *CaptionList;
      bool MouseCatched;
      TTransBitmapState TempState;
      int NormalHeight, SubHeight, SuperHeight;
      int SimWidth, SimHeight;   // Used for "SimulateNewWidth" and
                                 // "SimulateNewHeight"
      Graphics::TBitmap *BufferBitmap;   // Where we do all the graphics before
                                         // copying it to the canvas
      Graphics::TBitmap *MonoBitmap;   // Used for resolving the full
                                       // transparency
      bool NoMonoBitmap;
      Graphics::TBitmap *UnderCtrlBitmap;   // Contains what is under the
                                            // control
      bool ComputingUnderCtrlBitmap;
      Graphics::TBitmap *GradientBitmap;   // Used to keep the gradient, it is
                                           // pretty useful to make the painting
                                           // faster
      bool NoGradientBitmap;

      bool DoSoftRepaint;   // To know whether we should a soft paint or not, if
                            // not we update "UnderCtrlBitmap" before repainting

      // Methods to modify the different published properties

      void __fastcall SetAlignment(const TAlignment aValue);
      void __fastcall SetAllowAllUp(const bool aValue);
      void __fastcall SetAllowDown(const bool aValue);
      void __fastcall SetAllowFocus(const bool aValue);
      void __fastcall SetAllowFullTransparency(const bool aValue);
      void __fastcall SetAutoRefresh(const bool aValue);
      void __fastcall SetAllowSubAndSuperChars(const bool aValue);
      void __fastcall SetAutoSize(const bool aValue);
      void __fastcall SetAutoSizeParams(const TTransBitmapAutoSize aValue);
      void __fastcall SetBackgroundColor(const TColor aValue);
      void __fastcall SetBevelInner(const TPanelBevel aValue);
      void __fastcall SetBevelOuter(const TPanelBevel aValue);
      void __fastcall SetBevelWidth(const int aValue);
      void __fastcall SetBitmap(Graphics::TBitmap *aValue);
      void __fastcall SetBitmapDisplay(const TTransBitmapBitmapDisplay aValue);
      void __fastcall SetBitmapNumber(const TTransBitmapBitmapNumber aValue);
      void __fastcall SetBorderWidth(const TBorderWidth aValue);
      void __fastcall SetCaptionDisplay(const TTransBitmapCaptionDisplay aValue);
      void __fastcall SetCaptionNbLines(const int aValue);
      void __fastcall SetCaptionToBitmapDist(const int aValue);
      void __fastcall SetEscapeChar(const String aValue);
      void __fastcall SetFocusColor(const TColor aValue);
      void __fastcall SetFocusWidth(const int aValue);
      void __fastcall SetGradientColor(const TColor aValue);
      void __fastcall SetGradientEnabled(const bool aValue);
      void __fastcall SetGradientNbOfColors(const int aValue);
      void __fastcall SetGradientType(const TTransBitmapGradientType aValue);
      void __fastcall SetGroupIndex(const int aValue);
      void __fastcall SetLook(const TTransBitmapLook aValue);
      void __fastcall SetState(const TTransBitmapState aValue);
      void __fastcall SetStretch(const Boolean aValue);
      void __fastcall SetSubChar(const String aValue);
      void __fastcall SetSubCharDist(const int aValue);
      void __fastcall SetSubFontSizeDiff(const int aValue);
      void __fastcall SetSuperChar(const String aValue);
      void __fastcall SetSuperCharDist(const int aValue);
      void __fastcall SetSuperFontSizeDiff(const int aValue);

      // Methods used for internal purpose

      void __fastcall DisplayBackground(const TRect &aRegion);
      void __fastcall DisplayBitmap(const TRect &aRegion);
      void __fastcall GetSubBitmap(Graphics::TBitmap *aDestBmp,
                                   const TRect &aDestRegion, TRect &aSrcRegion);
      void __fastcall DisplayCaption(const TRect &aRegion);
      void __fastcall DisplayLook(TRect &aRegion);
      void __fastcall ComputeMonoBitmap();
      void __fastcall ComputeUnderCtrlBitmap();
      void __fastcall ComputeGradientBitmap(const int &aWidth,
                                            const int &aHeight);
      void __fastcall HorizGradientFill(const int &aWidth,
                                        const int &aHeight,
                                        const int BeginRGB1[3],
                                        const int RangeRGB1[3],
                                        const int BeginRGB2[3],
                                        const int RangeRGB2[3]);
      void __fastcall VertGradientFill(const int &aWidth,
                                       const int &aHeight,
                                       const int BeginRGB1[3],
                                       const int RangeRGB1[3],
                                       const int BeginRGB2[3],
                                       const int RangeRGB2[3]);
      bool __fastcall OverControl(const int &aX, const int &aY);
      bool __fastcall OverControl(const TPoint &aPoint);
      void __fastcall AdjustColors(const TPanelBevel &aBevel, TColor &aTopColor,
                                   TColor &aBottomColor);
      void __fastcall GetWorkableRegion(TRect &aRegion);
      int __fastcall CrtWidth(const String &aCaptionString, const int &aLimit,
                              const int &aCaptionLength,
                              const bool &aLastCaptionLine);
      String __fastcall GetACaptionLine(String &aCaptionString,
                                        const int &aRegionWidth,
                                        const bool &aLastCaptionLine);
      void __fastcall ComputeCaptionSize(const bool &aHeightLimit,
                                         const bool &aSimulation);
      void __fastcall OptimalSize(int &aWidth, int &aHeight);
      void __fastcall CheckSize(const bool &aSimulation = false);
      void __fastcall GetMaxima(int &aMaxWidth, int &aMaxHeight);
      void __fastcall GetBitmapWidth();
      void __fastcall GetBitmapPos(const TRect &aRegion, TPoint &aBmpPt);
      void __fastcall GetCaptionPos(const TRect &aRegion, TPoint &aTxtPt);
      void __fastcall CanvasTextOut(int &aX, int &aY, String &aText);
      int __fastcall CanvasTextWidth(String &aText);
      int __fastcall CanvasTextHeight(String &aText);
      bool __fastcall HasSubOrSuperChars(String &aText,
                                         const bool &CheckForSubChars,
                                         const bool &CheckForSuperChars);
      bool __fastcall HasSubChars(String &aText);
      bool __fastcall HasSuperChars(String &aText);
      void __fastcall UpdateRectSize(TRect &aRegion, const int aDiff);

      void __fastcall Initialize(const bool &aDynamic);

      void __fastcall UpdateExclusive();

      inline TTransBitmapState __fastcall RealState() { return MouseCatched?TempState:FState; }
      inline void __fastcall SoftRepaint() { DoSoftRepaint = true; Repaint(); }
   protected:
      // Method to draw the control

      virtual void __fastcall Paint();

      // Methods handling different messages (from Windows or not)

      void __fastcall CMFontChanged(TMessage &aMsg);
      void __fastcall CMTextChanged(TMessage &aMsg);
      void __fastcall CMMouseEnter(TMessage &aMsg);
      void __fastcall CMMouseLeave(TMessage &aMsg);
      void __fastcall CMButtonPressed(TMessage &aMsg);

      // Methods to handle the mouse click on the control

#if (__BORLANDC__ >= 0x0530)   // i.e. NOT BCB 1.0
      __declspec(dynamic) void __fastcall MouseDown(TMouseButton aButton,
                                                    TShiftState aShift, int aX, int aY);
      __declspec(dynamic) void __fastcall MouseMove(TShiftState aShift, int aX, int aY);
      __declspec(dynamic) void __fastcall MouseUp(TMouseButton aButton, TShiftState aShift,
                                                  int aX, int aY);
#else
      virtual void __fastcall MouseDown(TMouseButton aButton,
                                        TShiftState aShift, int aX, int aY);
      virtual void __fastcall MouseMove(TShiftState aShift, int aX, int aY);
      virtual void __fastcall MouseUp(TMouseButton aButton, TShiftState aShift,
                                      int aX, int aY);
#endif

      // Special internal events

      TNotifyEvent FOnMouseEnter;   // Generated when the mouse enter the
                                    // control (cf. CM_MOUSEENTER)
      TNotifyEvent FOnMouseLeave;   // Generated when the mouse enter the
                                    // control (cf. CM_MOUSELEAVE)

   public:
      // Constructor & Destructor

      virtual __fastcall TTransBitmap(TComponent *aOwner);
      __fastcall TTransBitmap(TComponent *aOwner, const bool &aDynamic);
      virtual __fastcall ~TTransBitmap();

      int __fastcall SimulateNewWidth(const int &aNewWidth);
      int __fastcall SimulateNewHeight(const int &aNewHeight);

      void __fastcall ForceRefresh();

   __published:
      // Read-only properties

      __property bool HasFocus = {read=FHasFocus};

      // Published properties

      __property Align;
      __property TAlignment Alignment = {read=FAlignment, write=SetAlignment, default=taCenter};
      __property bool AllowAllUp = {read=FAllowAllUp, write=SetAllowAllUp, default=false};
      __property bool AllowDown = {read=FAllowDown, write=SetAllowDown, default=false};
      __property bool AllowFocus = {read=FAllowFocus, write=SetAllowFocus, default=false};
      __property bool AllowFullTransparency = {read=FAllowFullTransparency, write=SetAllowFullTransparency, default=false};
      __property bool AutoRefresh = {read=FAutoRefresh, write=SetAutoRefresh, default=true};
      __property bool AllowSubAndSuperChars = {read=FAllowSubAndSuperChars, write=SetAllowSubAndSuperChars, default=false};
      __property bool AutoSize = {read=FAutoSize, write=SetAutoSize, default=false};
      __property TTransBitmapAutoSize AutoSizeParams = {read=FAutoSizeParams, write=SetAutoSizeParams, default=asBoth};
      __property TColor BackgroundColor = {read=FBackgroundColor, write=SetBackgroundColor, default=clBtnFace};
      __property Graphics::TBitmap *Bitmap = {read=FBitmap, write=SetBitmap, nodefault};
      __property TTransBitmapBitmapDisplay BitmapDisplay = {read=FBitmapDisplay, write=SetBitmapDisplay, default=bdMiddleCenter};
      __property TTransBitmapBitmapNumber BitmapNumber = {read=FBitmapNumber, write=SetBitmapNumber, default=bn1};
      __property TPanelBevel BevelInner = {read=FBevelInner, write=SetBevelInner, default=bvNone};
      __property TPanelBevel BevelOuter = {read=FBevelOuter, write=SetBevelOuter, default=bvRaised};
      __property int BevelWidth = {read=FBevelWidth, write=SetBevelWidth, default=1};
      __property TBorderWidth BorderWidth = {read=FBorderWidth, write=SetBorderWidth, default=0};
      __property Caption;
      __property TTransBitmapCaptionDisplay CaptionDisplay = {read=FCaptionDisplay, write=SetCaptionDisplay, default=cdBitmapTop};
      __property int CaptionNbLines = {read=FCaptionNbLines, write=SetCaptionNbLines, default=3};
      __property int CaptionToBitmapDist = {read=FCaptionToBitmapDist, write=SetCaptionToBitmapDist, default=7};
      __property DragCursor;
      __property DragMode;
      __property Enabled;
      __property String EscapeChar = {read=FEscapeChar, write=SetEscapeChar, nodefault};
      __property TColor FocusColor = {read=FFocusColor, write=SetFocusColor, default=clHighlight};
      __property int FocusWidth = {read=FFocusWidth, write=SetFocusWidth, default=2};
      __property Font;
      __property TColor GradientColor = {read=FGradientColor, write=SetGradientColor, default=clBlack};
      __property bool GradientEnabled = {read=FGradientEnabled, write=SetGradientEnabled, default=false};
      __property int GradientNbOfColors = {read=FGradientNbOfColors, write=SetGradientNbOfColors, default=0};
      __property TTransBitmapGradientType GradientType = {read=FGradientType, write=SetGradientType, default=grLeftToRight};
      __property int GroupIndex = {read=FGroupIndex, write=SetGroupIndex, default=0};
      __property TTransBitmapLook Look = {read=FLook, write=SetLook, default=loButton};
      __property ParentFont;
      __property ParentShowHint;
      __property PopupMenu;
      __property ShowHint;
      __property TTransBitmapState State = {read=FState, write=SetState, default=stUp};
      __property bool Stretch = {read=FStretch, write=SetStretch, default=false};
      __property String SubChar = {read=FSubChar, write=SetSubChar, nodefault};
      __property int SubCharDist = {read=FSubCharDist, write=SetSubCharDist, default=0};
      __property int SubFontSizeDiff = {read=FSubFontSizeDiff, write=SetSubFontSizeDiff, default=3};
      __property String SuperChar = {read=FSuperChar, write=SetSuperChar, nodefault};
      __property int SuperCharDist = {read=FSuperCharDist, write=SetSuperCharDist, default=0};
      __property int SuperFontSizeDiff = {read=FSuperFontSizeDiff, write=SetSuperFontSizeDiff, default=3};
      __property Visible;

      // Available events

      __property OnClick;
      __property OnDragDrop;
      __property OnDragOver;
      __property OnEndDrag;
      __property OnMouseDown;
      __property OnMouseMove;
      __property OnMouseUp;
      __property OnStartDrag;

      // TTransBitmap events

      __property TNotifyEvent OnMouseEnter = {read=FOnMouseEnter, write=FOnMouseEnter};
      __property TNotifyEvent OnMouseLeave = {read=FOnMouseLeave, write=FOnMouseLeave};

   BEGIN_MESSAGE_MAP
      MESSAGE_HANDLER(CM_FONTCHANGED, TMessage, CMFontChanged)
      MESSAGE_HANDLER(CM_TEXTCHANGED, TMessage, CMTextChanged)
      MESSAGE_HANDLER(CM_MOUSEENTER, TMessage, CMMouseEnter)
      MESSAGE_HANDLER(CM_MOUSELEAVE, TMessage, CMMouseLeave)
      MESSAGE_HANDLER(CM_BUTTONPRESSED, TMessage, CMButtonPressed)
   END_MESSAGE_MAP(TGraphicControl)
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////

